@@ -102,6 +102,7 @@ struct gpgsm_object_s
       int linesize;
       int linelen;
     } attic;
+    int any; /* any data line seen */
   } colon; 
 
   struct GpgmeIOCbs io_cbs;
@@ -220,8 +221,10 @@ map_assuan_error (AssuanError err)
     case ASSUAN_Unsupported_Algorithm:
       return mk_error (Not_Implemented);  /* XXX Argh.  */
       
-      /* These are errors internal to GPGME.  */
     case ASSUAN_No_Data_Available:
+      return mk_error (EOF);
+      
+      /* These are errors internal to GPGME.  */
     case ASSUAN_No_Input:
     case ASSUAN_No_Output:
     case ASSUAN_Invalid_Command:
@@ -324,6 +327,7 @@ _gpgme_gpgsm_new (GpgsmObject *r_gpgsm)
   gpgsm->colon.attic.line = 0;
   gpgsm->colon.attic.linesize = 0;
   gpgsm->colon.attic.linelen = 0;
+  gpgsm->colon.any = 0;
 
   gpgsm->io_cbs.add = NULL;
   gpgsm->io_cbs.add_priv = NULL;
@@ -543,7 +547,7 @@ _gpgme_gpgsm_release (GpgsmObject gpgsm)
 }
 
 /* Forward declaration.  */
-static GpgStatusCode parse_status (const char *name);
+static GpgmeStatusCode parse_status (const char *name);
 
 static GpgmeError
 gpgsm_assuan_simple_command (ASSUAN_CONTEXT ctx, char *cmd, GpgStatusHandler status_fnc,
@@ -578,7 +582,7 @@ gpgsm_assuan_simple_command (ASSUAN_CONTEXT ctx, char *cmd, GpgStatusHandler sta
 	       && line[0] == 'S' && line[1] == ' ')
 	{
 	  char *rest;
-	  GpgStatusCode r;
+	  GpgmeStatusCode r;
 
 	  rest = strchr (line + 2, ' ');
 	  if (!rest)
@@ -667,8 +671,63 @@ _gpgme_gpgsm_op_decrypt (GpgsmObject gpgsm, GpgmeData ciph, GpgmeData plain)
 GpgmeError
 _gpgme_gpgsm_op_delete (GpgsmObject gpgsm, GpgmeKey key, int allow_secret)
 {
-  /* FIXME */
-  return mk_error (Not_Implemented);
+  char *fpr = (char *) gpgme_key_get_string_attr (key, GPGME_ATTR_FPR, NULL, 0);
+  char *linep = fpr;
+  char *line;
+  int length = 8;	/* "DELKEYS " */
+
+  if (!fpr)
+    return mk_error (Invalid_Key);
+
+  while (*linep)
+    {
+      length++;
+      if (*linep == '%' || *linep == ' ' || *linep == '+')
+	length += 2;
+      linep++;
+    }
+  length++;
+
+  line = xtrymalloc (length);
+  if (!line)
+    return mk_error (Out_Of_Core);
+
+  strcpy (line, "DELKEYS ");
+  linep = &line[8];
+
+  while (*fpr)
+    {
+      switch (*fpr)
+	{
+	case '%':
+	  *(linep++) = '%';
+	  *(linep++) = '2';
+	  *(linep++) = '5';
+	  break;
+	case ' ':
+	  *(linep++) = '%';
+	  *(linep++) = '2';
+	  *(linep++) = '0';
+	  break;
+	case '+':
+	  *(linep++) = '%';
+	  *(linep++) = '2';
+	  *(linep++) = 'B';
+	  break;
+	default:
+	  *(linep++) = *fpr;
+	  break;
+	}
+      fpr++;
+    }
+  *linep = '\0';
+
+  gpgsm->command = line;
+  _gpgme_io_close (gpgsm->output_cb.fd);
+  _gpgme_io_close (gpgsm->input_cb.fd);
+  _gpgme_io_close (gpgsm->message_cb.fd);
+
+  return 0;
 }
 
 
@@ -715,7 +774,7 @@ gpgsm_set_recipients (GpgsmObject gpgsm, GpgmeRecipients recp)
     }
   xfree (line);
   if (!valid_recipients && gpgsm->status.fnc)
-    gpgsm->status.fnc (gpgsm->status.fnc_value, STATUS_NO_RECP, "");
+    gpgsm->status.fnc (gpgsm->status.fnc_value, GPGME_STATUS_NO_RECP, "");
   return 0;
 }
 
@@ -1019,6 +1078,8 @@ _gpgme_gpgsm_op_sign (GpgsmObject gpgsm, GpgmeData in, GpgmeData out,
 {
   GpgmeError err;
   char *assuan_cmd;
+  int i;
+  GpgmeKey key;
 
   if (!gpgsm)
     return mk_error (Invalid_Value);
@@ -1030,11 +1091,36 @@ _gpgme_gpgsm_op_sign (GpgsmObject gpgsm, GpgmeData in, GpgmeData out,
 
   if (asprintf (&assuan_cmd, "OPTION include-certs %i", include_certs) < 0)
     return mk_error (Out_Of_Core);
-  err = gpgsm_assuan_simple_command (gpgsm->assuan_ctx, assuan_cmd, NULL, NULL);
+  err = gpgsm_assuan_simple_command (gpgsm->assuan_ctx, assuan_cmd, NULL,NULL);
   free (assuan_cmd);
   if (err)
     return err;
 
+  /* We must do a reset becuase we need to reset the list of signers.  Note
+     that RESET does not reset OPTION commands. */
+  err = gpgsm_assuan_simple_command (gpgsm->assuan_ctx, "RESET", NULL, NULL);
+  if (err)
+    return err;
+
+  for (i = 0; (key = gpgme_signers_enum (ctx, i)); i++)
+    {
+      const char *s = gpgme_key_get_string_attr (key, GPGME_ATTR_FPR,
+						 NULL, 0);
+      if (s && strlen (s) < 80)
+	{
+          char buf[100];
+
+          strcpy (stpcpy (buf, "SIGNER "), s);
+          err = gpgsm_assuan_simple_command (gpgsm->assuan_ctx, buf,
+                                             NULL, NULL);
+	}
+      else
+        err = GPGME_Invalid_Key;
+      gpgme_key_unref (key);
+      if (err) 
+        return err;
+    }
+
   gpgsm->input_cb.data = in;
   err = gpgsm_set_fd (gpgsm->assuan_ctx, "INPUT", gpgsm->input_fd_server,
                       map_input_enc (gpgsm->input_cb.data));
@@ -1109,7 +1195,7 @@ status_cmp (const void *ap, const void *bp)
 }
 
 
-static GpgStatusCode
+static GpgmeStatusCode
 parse_status (const char *name)
 {
   struct status_table_s t, *r;
@@ -1159,7 +1245,17 @@ gpgsm_status_handler (void *opaque, int fd)
 	    }
 
 	  if (gpgsm->status.fnc)
-	    gpgsm->status.fnc (gpgsm->status.fnc_value, STATUS_EOF, "");
+	    gpgsm->status.fnc (gpgsm->status.fnc_value, GPGME_STATUS_EOF, "");
+	  if (gpgsm->colon.fnc && gpgsm->colon.any )
+            {
+              /* We must tell a colon fucntion about the EOF. We do
+                 this only when we have seen any data lines.  Note
+                 that this inlined use of colon data lines will
+                 eventually be changed into using a regular data
+                 channel. */
+              gpgsm->colon.any = 0;
+              gpgsm->colon.fnc (gpgsm->colon.fnc_value, NULL);
+            }
 
 	  /* XXX: Try our best to terminate the connection.  */
 	  if (err)
@@ -1222,6 +1318,7 @@ gpgsm_status_handler (void *opaque, int fd)
 		  /* Terminate the pending line, pass it to the colon
 		     handler and reset it.  */
 
+                  gpgsm->colon.any = 1;
 		  if (*alinelen > 1 && *(dst - 1) == '\r')
 		    dst--;
 		  *dst = '\0';
@@ -1239,7 +1336,7 @@ gpgsm_status_handler (void *opaque, int fd)
 	  && line[0] == 'S' && line[1] == ' ')
 	{
 	  char *rest;
-	  GpgStatusCode r;
+	  GpgmeStatusCode r;
 
 	  rest = strchr (line + 2, ' ');
 	  if (!rest)
@@ -1281,6 +1378,7 @@ _gpgme_gpgsm_set_colon_line_handler (GpgsmObject gpgsm,
 
   gpgsm->colon.fnc = fnc;
   gpgsm->colon.fnc_value = fnc_value;
+  gpgsm->colon.any = 0;
 }
 
 
@@ -1288,14 +1386,14 @@ static GpgmeError
 _gpgme_gpgsm_add_io_cb (GpgsmObject gpgsm, iocb_data_t *iocbd,
 			GpgmeIOCb handler)
 {
-  GpgmeError err = 0;
+  GpgmeError err;
 
-  iocbd->tag = (*gpgsm->io_cbs.add) (gpgsm->io_cbs.add_priv,
-				     iocbd->fd, iocbd->dir,
-				     handler, iocbd->data);
-  if (!iocbd->tag)
-    err = mk_error (General_Error);
-  if (!err && !iocbd->dir)
+  err = (*gpgsm->io_cbs.add) (gpgsm->io_cbs.add_priv,
+			      iocbd->fd, iocbd->dir,
+			      handler, iocbd->data, &iocbd->tag);
+  if (err)
+    return err;
+  if (!iocbd->dir)
     /* FIXME Kludge around poll() problem.  */
     err = _gpgme_io_set_nonblocking (iocbd->fd);
   return err;
@@ -1336,6 +1434,12 @@ _gpgme_gpgsm_set_io_cbs (GpgsmObject gpgsm, struct GpgmeIOCbs *io_cbs)
   gpgsm->io_cbs = *io_cbs;
 }
 
+void
+_gpgme_gpgsm_io_event (GpgsmObject gpgsm, GpgmeEventIO type, void *type_data)
+{
+  if (gpgsm->io_cbs.event)
+    (*gpgsm->io_cbs.event) (gpgsm->io_cbs.event_priv, type, type_data);
+}
 
 #else	/* ENABLE_GPGSM */
 
@@ -1484,4 +1588,9 @@ _gpgme_gpgsm_set_io_cbs (GpgsmObject gpgsm, struct GpgmeIOCbs *io_cbs)
 {
 }
 
+void
+_gpgme_gpgsm_io_event (GpgsmObject gpgsm, GpgmeEventIO type, void *type_data)
+{
+}
+
 #endif	/* ! ENABLE_GPGSM */
