@@ -32,6 +32,8 @@
 
 struct import_result_s
 {
+  int nr_imported;
+  int nr_considered;
   GpgmeData xmlinfo;
 };
 
@@ -50,11 +52,13 @@ _gpgme_release_import_result (ImportResult result)
    the data buffer.  With args of NULL the xml structure is
    closed.  */
 static void
-append_xml_impinfo (GpgmeData *rdh, GpgStatusCode code, char *args)
+append_xml_impinfo (GpgmeData *rdh, GpgmeStatusCode code, char *args)
 {
 #define MAX_IMPORTED_FIELDS 14
   static const char *const imported_fields[MAX_IMPORTED_FIELDS]
     = { "keyid", "username", 0 };
+  static const char *const imported_fields_x509[MAX_IMPORTED_FIELDS]
+    = { "fpr", 0 };
   static const char *const import_res_fields[MAX_IMPORTED_FIELDS]
     = { "count", "no_user_id", "imported", "imported_rsa",
 	"unchanged", "n_uids", "n_subk", "n_sigs", "s_sigsn_revoc",
@@ -65,14 +69,14 @@ append_xml_impinfo (GpgmeData *rdh, GpgStatusCode code, char *args)
   int i;
 
   /* Verify that we can use the args.  */
-  if (code != STATUS_EOF)
+  if (code != GPGME_STATUS_EOF)
     {
       if (!args)
 	return;
 
-      if (code == STATUS_IMPORTED)
+      if (code == GPGME_STATUS_IMPORTED)
 	field_name = imported_fields;
-      else if (code == STATUS_IMPORT_RES)
+      else if (code == GPGME_STATUS_IMPORT_RES)
 	field_name = import_res_fields;
       else
 	return;
@@ -88,6 +92,11 @@ append_xml_impinfo (GpgmeData *rdh, GpgStatusCode code, char *args)
 	      *args++ = '\0';
 	    }
 	}
+      
+      /* gpgsm does not print a useful user ID and uses a fingerprint
+         instead of the key ID. */
+      if (code == GPGME_STATUS_IMPORTED && field[0] && strlen (field[0]) > 16)
+        field_name = imported_fields_x509;
     }
 
   /* Initialize the data buffer if necessary.  */
@@ -101,39 +110,39 @@ append_xml_impinfo (GpgmeData *rdh, GpgStatusCode code, char *args)
   else
     dh = *rdh;
     
-  if (code == STATUS_EOF)
+  if (code == GPGME_STATUS_EOF)
     {
       /* Just close the XML containter.  */
       _gpgme_data_append_string (dh, "</GnupgOperationInfo>\n");
     }
   else
     {
-      if (code == STATUS_IMPORTED)
+      if (code == GPGME_STATUS_IMPORTED)
 	_gpgme_data_append_string (dh, "  <import>\n");
-      else if (code == STATUS_IMPORT_RES)
+      else if (code == GPGME_STATUS_IMPORT_RES)
 	_gpgme_data_append_string (dh, "  <importResult>\n");
 
       for (i = 0; field_name[i]; i++)
 	{
 	  _gpgme_data_append_string (dh, "    <");
-	  _gpgme_data_append_string (dh, field_name[i]);
+          _gpgme_data_append_string (dh, field_name[i]);
 	  _gpgme_data_append_string (dh, ">");
-	  _gpgme_data_append_string (dh, field[i]);
+	  _gpgme_data_append_string_for_xml (dh, field[i]);
 	  _gpgme_data_append_string (dh, "</");
 	  _gpgme_data_append_string (dh, field_name[i]);
 	  _gpgme_data_append_string (dh, ">\n");
 	}
 
-      if (code == STATUS_IMPORTED)
+      if (code == GPGME_STATUS_IMPORTED)
 	_gpgme_data_append_string (dh, "  </import>\n");
-      else if (code == STATUS_IMPORT_RES)
+      else if (code == GPGME_STATUS_IMPORT_RES)
 	_gpgme_data_append_string (dh, "  </importResult>\n");
     }
 }
 
 
 static void
-import_status_handler (GpgmeCtx ctx, GpgStatusCode code, char *args)
+import_status_handler (GpgmeCtx ctx, GpgmeStatusCode code, char *args)
 {
   if (ctx->error)
     return;
@@ -141,7 +150,7 @@ import_status_handler (GpgmeCtx ctx, GpgStatusCode code, char *args)
 
   switch (code)
     {
-    case STATUS_EOF:
+    case GPGME_STATUS_EOF:
       if (ctx->result.import->xmlinfo)
         {
           append_xml_impinfo (&ctx->result.import->xmlinfo, code, NULL);
@@ -151,8 +160,13 @@ import_status_handler (GpgmeCtx ctx, GpgStatusCode code, char *args)
       /* XXX Calculate error value.  */
       break;
 
-    case STATUS_IMPORTED:
-    case STATUS_IMPORT_RES:
+    case GPGME_STATUS_IMPORTED:
+      ctx->result.import->nr_imported++;
+      append_xml_impinfo (&ctx->result.import->xmlinfo, code, args);
+      break;
+
+    case GPGME_STATUS_IMPORT_RES:
+      ctx->result.import->nr_considered = strtol (args, 0, 0);
       append_xml_impinfo (&ctx->result.import->xmlinfo, code, args);
       break;
 
@@ -208,16 +222,31 @@ gpgme_op_import_start (GpgmeCtx ctx, GpgmeData keydata)
  * gpgme_op_import:
  * @c: Context 
  * @keydata: Data object
+ * @nr: Will contain number of considered keys.
  * 
  * Import all key material from @keydata into the key database.
  * 
- * Return value: o on success or an error code.
+ * Return value: 0 on success or an error code.
  **/
 GpgmeError
-gpgme_op_import (GpgmeCtx ctx, GpgmeData keydata)
+gpgme_op_import_ext (GpgmeCtx ctx, GpgmeData keydata, int *nr)
 {
   GpgmeError err = _gpgme_op_import_start (ctx, 1, keydata);
   if (!err)
     err = _gpgme_wait_one (ctx);
+  if (!err && nr)
+    {
+      if (ctx->result.import)
+	*nr = ctx->result.import->nr_considered;
+      else
+	*nr = 0;
+    }
   return err;
 }
+
+GpgmeError
+gpgme_op_import (GpgmeCtx ctx, GpgmeData keydata)
+{
+  return gpgme_op_import_ext (ctx, keydata, 0);
+}
+
