@@ -100,8 +100,6 @@ struct gpg_object_s
   char **argv;  
   struct fd_data_map_s *fd_data_map;
 
-  int pid; /* we can't use pid_t because we don't use it in Windoze */
-
   /* stuff needed for pipemode */
   struct
   {
@@ -119,10 +117,14 @@ struct gpg_object_s
     int fd;
     int idx;		/* Index in fd_data_map */
     GpgmeData cb_data;   /* hack to get init the above idx later */
-    GpgStatusCode code;  /* last code */
+    GpgmeStatusCode code;  /* last code */
     char *keyword;       /* what has been requested (malloced) */
     GpgCommandHandler fnc; 
     void *fnc_value;
+    /* The kludges never end.  This is used to couple command handlers
+       with output data in edit key mode.  */
+    GpgmeData linked_data;
+    int linked_idx;
   } cmd;
 
   struct GpgmeIOCbs io_cbs;
@@ -212,8 +214,8 @@ close_notify_handler (int fd, void *opaque)
 	    break;
 	  }
     }
-  if (!not_done && gpg->io_cbs.event)
-    (*gpg->io_cbs.event) (gpg->io_cbs.event_priv, GPGME_EVENT_DONE, NULL);
+  if (!not_done)
+    _gpgme_gpg_io_event (gpg, GPGME_EVENT_DONE, NULL);
 }
 
 const char *
@@ -257,8 +259,8 @@ _gpgme_gpg_new (GpgObject *r_gpg)
   gpg->colon.fd[1] = -1;
   gpg->cmd.fd = -1;
   gpg->cmd.idx = -1;
-
-  gpg->pid = -1;
+  gpg->cmd.linked_data = NULL;
+  gpg->cmd.linked_idx = -1;
 
   /* Allocate the read buffer for the status pipe.  */
   gpg->status.bufsize = 1024;
@@ -292,6 +294,8 @@ _gpgme_gpg_new (GpgObject *r_gpg)
     _gpgme_gpg_add_arg (gpg, buf);
   }
   _gpgme_gpg_add_arg (gpg, "--no-tty");
+  _gpgme_gpg_add_arg (gpg, "--charset");
+  _gpgme_gpg_add_arg (gpg, "utf8");
 
  leave:
   if (rc)
@@ -337,10 +341,7 @@ _gpgme_gpg_release (GpgObject gpg)
   free_fd_data_map (gpg->fd_data_map);
   if (gpg->cmd.fd != -1)
     _gpgme_io_close (gpg->cmd.fd);
-  if (gpg->pid != -1)
-    _gpgme_engine_add_child_to_reap_list (gpg, sizeof *gpg, gpg->pid);
-  else
-    xfree (gpg);
+  xfree (gpg);
 }
 
 void
@@ -516,27 +517,29 @@ _gpgme_gpg_set_simple_line_handler ( GpgObject gpg,
  */
 
 GpgmeError
-_gpgme_gpg_set_command_handler ( GpgObject gpg,
-                                 GpgCommandHandler fnc, void *fnc_value ) 
+_gpgme_gpg_set_command_handler (GpgObject gpg,
+				GpgCommandHandler fnc, void *fnc_value,
+				GpgmeData linked_data)
 {
-    GpgmeData tmp;
-    GpgmeError err;
+  GpgmeData tmp;
+  GpgmeError err;
 
-    assert (gpg);
-    if (gpg->pm.active)
-        return 0;
-
-    err = gpgme_data_new_with_read_cb ( &tmp, command_cb, gpg );
-    if (err)
-        return err;
-        
-    _gpgme_gpg_add_arg ( gpg, "--command-fd" );
-    _gpgme_gpg_add_data (gpg, tmp, -2);
-    gpg->cmd.cb_data = tmp;
-    gpg->cmd.fnc = fnc;
-    gpg->cmd.fnc_value = fnc_value;
-    gpg->cmd.used = 1;
+  assert (gpg);
+  if (gpg->pm.active)
     return 0;
+
+  err = gpgme_data_new_with_read_cb (&tmp, command_cb, gpg);
+  if (err)
+    return err;
+        
+  _gpgme_gpg_add_arg ( gpg, "--command-fd" );
+  _gpgme_gpg_add_data (gpg, tmp, -2);
+  gpg->cmd.cb_data = tmp;
+  gpg->cmd.fnc = fnc;
+  gpg->cmd.fnc_value = fnc_value;
+  gpg->cmd.linked_data = linked_data;
+  gpg->cmd.used = 1;
+  return 0;
 }
 
 
@@ -761,10 +764,18 @@ build_argv (GpgObject gpg)
 	  }
 
 	  /* Hack to get hands on the fd later.  */
-	  if (gpg->cmd.used && gpg->cmd.cb_data == a->data)
+	  if (gpg->cmd.used)
 	    {
-	      assert (gpg->cmd.idx == -1);
-	      gpg->cmd.idx = datac;
+	      if (gpg->cmd.cb_data == a->data)
+		{
+		  assert (gpg->cmd.idx == -1);
+		  gpg->cmd.idx = datac;
+		}
+	      else if (gpg->cmd.linked_data == a->data)
+		{
+		  assert (gpg->cmd.linked_idx == -1);
+		  gpg->cmd.linked_idx = datac;
+		}
 	    }
 
 	  fd_data_map[datac].data = a->data;
@@ -807,12 +818,12 @@ static GpgmeError
 _gpgme_gpg_add_io_cb (GpgObject gpg, int fd, int dir,
 		      GpgmeIOCb handler, void *data, void **tag)
 {
-  GpgmeError err = 0;
+  GpgmeError err;
 
-  *tag = (*gpg->io_cbs.add) (gpg->io_cbs.add_priv, fd, dir, handler, data);
-  if (!tag)
-    err = mk_error (General_Error);
-  if (!err && !dir)
+  err = (*gpg->io_cbs.add) (gpg->io_cbs.add_priv, fd, dir, handler, data, tag);
+  if (err)
+    return err;
+  if (!dir)
     /* FIXME Kludge around poll() problem.  */
     err = _gpgme_io_set_nonblocking (fd);
   return err;
@@ -823,7 +834,7 @@ _gpgme_gpg_spawn (GpgObject gpg, void *opaque)
 {
   GpgmeError rc;
   int i, n;
-  int pid;
+  int status;
   struct spawn_fd_item_s *fd_child_list, *fd_parent_list;
 
   if (!gpg)
@@ -898,13 +909,12 @@ _gpgme_gpg_spawn (GpgObject gpg, void *opaque)
   fd_parent_list[n].fd = -1;
   fd_parent_list[n].dup_to = -1;
 
-  pid = _gpgme_io_spawn (_gpgme_get_gpg_path (),
-			 gpg->argv, fd_child_list, fd_parent_list);
+  status = _gpgme_io_spawn (_gpgme_get_gpg_path (),
+			    gpg->argv, fd_child_list, fd_parent_list);
   xfree (fd_child_list);
-  if (pid == -1)
+  if (status == -1)
     return mk_error (Exec_Error);
 
-  gpg->pid = pid;
   if (gpg->pm.used)
     gpg->pm.active = 1;
 
@@ -999,86 +1009,118 @@ status_cmp (const void *ap, const void *bp)
  * with status line code we know about and skip all other stuff
  * without buffering (i.e. without extending the buffer).  */
 static GpgmeError
-read_status ( GpgObject gpg )
+read_status (GpgObject gpg)
 {
-    char *p;
-    int nread;
-    size_t bufsize = gpg->status.bufsize; 
-    char *buffer = gpg->status.buffer;
-    size_t readpos = gpg->status.readpos; 
+  char *p;
+  int nread;
+  size_t bufsize = gpg->status.bufsize; 
+  char *buffer = gpg->status.buffer;
+  size_t readpos = gpg->status.readpos; 
 
-    assert (buffer);
-    if (bufsize - readpos < 256) { 
-        /* need more room for the read */
-        bufsize += 1024;
-        buffer = xtryrealloc (buffer, bufsize);
-        if ( !buffer ) 
-            return mk_error (Out_Of_Core);
-    }
-    
-
-    nread = _gpgme_io_read ( gpg->status.fd[0],
-                             buffer+readpos, bufsize-readpos );
-    if (nread == -1)
-        return mk_error(Read_Error);
-
-    if (!nread) {
-        gpg->status.eof = 1;
-        if (gpg->status.fnc)
-            gpg->status.fnc ( gpg->status.fnc_value, STATUS_EOF, "" );
-        return 0;
+  assert (buffer);
+  if (bufsize - readpos < 256)
+    { 
+      /* Need more room for the read.  */
+      bufsize += 1024;
+      buffer = xtryrealloc (buffer, bufsize);
+      if (!buffer)
+	return mk_error (Out_Of_Core);
     }
 
-    while (nread > 0) {
-        for (p = buffer + readpos; nread; nread--, p++) {
-            if ( *p == '\n' ) {
-                /* (we require that the last line is terminated by a LF) */
-                *p = 0;
-                if (!strncmp (buffer, "[GNUPG:] ", 9 )
-                    && buffer[9] >= 'A' && buffer[9] <= 'Z' ) {
-                    struct status_table_s t, *r;
-                    char *rest;
+  nread = _gpgme_io_read (gpg->status.fd[0],
+			  buffer + readpos, bufsize-readpos);
+  if (nread == -1)
+    return mk_error(Read_Error);
 
-                    rest = strchr (buffer+9, ' ');
-                    if ( !rest )
-                        rest = p; /* set to an empty string */
-                    else
-                        *rest++ = 0;
+  if (!nread)
+    {
+      gpg->status.eof = 1;
+      if (gpg->status.fnc)
+	gpg->status.fnc (gpg->status.fnc_value, GPGME_STATUS_EOF, "");
+      return 0;
+    }
+
+  while (nread > 0)
+    {
+      for (p = buffer + readpos; nread; nread--, p++)
+	{
+	  if (*p == '\n')
+	    {
+	      /* (we require that the last line is terminated by a LF) */
+	      *p = 0;
+	      if (!strncmp (buffer, "[GNUPG:] ", 9)
+		  && buffer[9] >= 'A' && buffer[9] <= 'Z')
+		{
+		  struct status_table_s t, *r;
+		  char *rest;
+
+		  rest = strchr (buffer + 9, ' ');
+		  if (!rest)
+		    rest = p; /* Set to an empty string.  */
+		  else
+		    *rest++ = 0;
                     
-                    t.name = buffer+9;
-                    /* (the status table as one extra element) */
-                    r = bsearch ( &t, status_table, DIM(status_table)-1,
-                                  sizeof t, status_cmp );
-                    if ( r ) {
-                        if ( gpg->cmd.used
-                             && ( r->code == STATUS_GET_BOOL
-                                  || r->code == STATUS_GET_LINE
-                                  || r->code == STATUS_GET_HIDDEN )) {
-                            gpg->cmd.code = r->code;
-                            xfree (gpg->cmd.keyword);
-                            gpg->cmd.keyword = xtrystrdup (rest);
-                            if ( !gpg->cmd.keyword )
-                                return mk_error (Out_Of_Core);
-                            /* this should be the last thing we have received
-                             * and the next thing will be that the command
-                             * handler does its action */
-                            if ( nread > 1 )
-                                DEBUG0 ("ERROR, unexpected data in read_status");
+		  t.name = buffer+9;
+		  /* (the status table has one extra element) */
+		  r = bsearch (&t, status_table, DIM(status_table) - 1,
+			       sizeof t, status_cmp);
+		  if (r)
+		    {
+		      if (gpg->cmd.used
+			  && (r->code == GPGME_STATUS_GET_BOOL
+			      || r->code == GPGME_STATUS_GET_LINE
+			      || r->code == GPGME_STATUS_GET_HIDDEN))
+			{
+			  gpg->cmd.code = r->code;
+			  xfree (gpg->cmd.keyword);
+			  gpg->cmd.keyword = xtrystrdup (rest);
+			  if (!gpg->cmd.keyword)
+			    return mk_error (Out_Of_Core);
+			  /* This should be the last thing we have
+			     received and the next thing will be that
+			     the command handler does its action.  */
+			  if (nread > 1)
+			    DEBUG0 ("ERROR, unexpected data in read_status");
 
-			    _gpgme_gpg_add_io_cb
-			      (gpg, gpg->cmd.fd,
-			       0, _gpgme_data_outbound_handler,
-			       gpg->fd_data_map[gpg->cmd.idx].data,
-			       &gpg->fd_data_map[gpg->cmd.idx].tag);
-			    gpg->fd_data_map[gpg->cmd.idx].fd = gpg->cmd.fd;
-			    gpg->cmd.fd = -1;
+			  /* Before we can actually add the command
+			     fd, we might have to flush the linked
+			     output data pipe.  */
+			  if (gpg->cmd.linked_idx != -1
+			      && gpg->fd_data_map[gpg->cmd.linked_idx].fd != -1)
+			    {
+			      struct io_select_fd_s fds;
+			      fds.fd = gpg->fd_data_map[gpg->cmd.linked_idx].fd;
+			      fds.for_read = 1;
+			      fds.for_write = 0;
+			      fds.frozen = 0;
+			      fds.opaque = NULL;
+			      do
+				{
+				  fds.signaled = 0;
+				  _gpgme_io_select (&fds, 1, 1);
+				  if (fds.signaled)
+				    _gpgme_data_inbound_handler
+				      (gpg->cmd.linked_data, fds.fd);
+				}
+			      while (fds.signaled);
+			    }
+
+			  _gpgme_gpg_add_io_cb
+			    (gpg, gpg->cmd.fd,
+			     0, _gpgme_data_outbound_handler,
+			     gpg->fd_data_map[gpg->cmd.idx].data,
+			     &gpg->fd_data_map[gpg->cmd.idx].tag);
+			  gpg->fd_data_map[gpg->cmd.idx].fd = gpg->cmd.fd;
+			  gpg->cmd.fd = -1;
                         }
-                        else if ( gpg->status.fnc ) {
-                            gpg->status.fnc ( gpg->status.fnc_value, 
-                                              r->code, rest);
+		      else if (gpg->status.fnc)
+			{
+			  gpg->status.fnc (gpg->status.fnc_value, 
+					   r->code, rest);
                         }
                     
-                        if ( r->code == STATUS_END_STREAM ) {
+		      if (r->code == GPGME_STATUS_END_STREAM)
+			{
 			  if (gpg->cmd.used)
 			    {
 			      /* XXX We must check if there are any
@@ -1092,28 +1134,28 @@ read_status ( GpgObject gpg )
                         }
                     }
                 }
-                /* To reuse the buffer for the next line we have to
-                 * shift the remaining data to the buffer start and
-                 * restart the loop Hmmm: We can optimize this
-                 * function by looking forward in the buffer to see
-                 * whether a second complete line is available and in
-                 * this case avoid the memmove for this line.  */
-                nread--; p++;
-                if (nread)
-                    memmove (buffer, p, nread);
-                readpos = 0;
-                break; /* the for loop */
+	      /* To reuse the buffer for the next line we have to
+		 shift the remaining data to the buffer start and
+		 restart the loop Hmmm: We can optimize this function
+		 by looking forward in the buffer to see whether a
+		 second complete line is available and in this case
+		 avoid the memmove for this line.  */
+	      nread--; p++;
+	      if (nread)
+		memmove (buffer, p, nread);
+	      readpos = 0;
+	      break; /* the for loop */
             }
-            else
-                readpos++;
+	  else
+	    readpos++;
         }
     } 
 
-    /* Update the gpg object.  */
-    gpg->status.bufsize = bufsize;
-    gpg->status.buffer = buffer;
-    gpg->status.readpos = readpos;
-    return 0;
+  /* Update the gpg object.  */
+  gpg->status.bufsize = bufsize;
+  gpg->status.buffer = buffer;
+  gpg->status.readpos = readpos;
+  return 0;
 }
 
 
@@ -1397,26 +1439,6 @@ _gpgme_gpg_op_delete (GpgObject gpg, GpgmeKey key, int allow_secret)
 }
 
 
-static GpgmeError
-_gpgme_append_gpg_args_from_recipients (GpgObject gpg,
-					const GpgmeRecipients rset)
-{
-  GpgmeError err = 0;
-  struct user_id_s *r;
-
-  assert (rset);
-  for (r = rset->list; r; r = r->next)
-    {
-      err = _gpgme_gpg_add_arg (gpg, "-r");
-      if (!err)
-	_gpgme_gpg_add_arg (gpg, r->name);
-      if (err)
-	break;
-    }    
-  return err;
-}
-
-
 static GpgmeError
 _gpgme_append_gpg_args_from_signers (GpgObject gpg,
 				     GpgmeCtx ctx /* FIXME */)
@@ -1443,6 +1465,54 @@ _gpgme_append_gpg_args_from_signers (GpgObject gpg,
 }
 
 
+GpgmeError
+_gpgme_gpg_op_edit (GpgObject gpg, GpgmeKey key, GpgmeData out,
+		    GpgmeCtx ctx /* FIXME */)
+{
+  GpgmeError err;
+
+  err = _gpgme_gpg_add_arg (gpg, "--with-colons");
+  if (!err)
+    err = _gpgme_append_gpg_args_from_signers (gpg, ctx);
+  if (!err)
+  err = _gpgme_gpg_add_arg (gpg, "--edit-key");
+  if (!err)
+    err = _gpgme_gpg_add_data (gpg, out, 1);
+  if (!err)
+    err = _gpgme_gpg_add_arg (gpg, "--");
+  if (!err)
+    {
+      const char *s = gpgme_key_get_string_attr (key, GPGME_ATTR_FPR, NULL, 0);
+      if (!s)
+	err = mk_error (Invalid_Key);
+      else
+	err = _gpgme_gpg_add_arg (gpg, s);
+    }
+
+  return err;
+}
+
+
+static GpgmeError
+_gpgme_append_gpg_args_from_recipients (GpgObject gpg,
+					const GpgmeRecipients rset)
+{
+  GpgmeError err = 0;
+  struct user_id_s *r;
+
+  assert (rset);
+  for (r = rset->list; r; r = r->next)
+    {
+      err = _gpgme_gpg_add_arg (gpg, "-r");
+      if (!err)
+	_gpgme_gpg_add_arg (gpg, r->name);
+      if (err)
+	break;
+    }    
+  return err;
+}
+
+
 GpgmeError
 _gpgme_gpg_op_encrypt (GpgObject gpg, GpgmeRecipients recp,
 		       GpgmeData plain, GpgmeData ciph, int use_armor)
@@ -1599,7 +1669,12 @@ _gpgme_gpg_op_keylist (GpgObject gpg, const char *pattern, int secret_only,
   if (!err)
     err = _gpgme_gpg_add_arg (gpg, "--with-fingerprint");
   if (!err)
-    err = _gpgme_gpg_add_arg (gpg, secret_only ? "--list-secret-keys"
+    err = _gpgme_gpg_add_arg (gpg, "--with-fingerprint");
+  if (!err)
+    err = _gpgme_gpg_add_arg (gpg, 
+                              (keylist_mode & GPGME_KEYLIST_MODE_SIGS)?
+                              "--check-sigs" :
+                              secret_only ? "--list-secret-keys"
 			      : "--list-keys");
   
   /* Tell the gpg object about the data */
@@ -1626,6 +1701,8 @@ _gpgme_gpg_op_keylist_ext (GpgObject gpg, const char *pattern[],
     err = _gpgme_gpg_add_arg (gpg, "--fixed-list-mode");
   if (!err)
     err = _gpgme_gpg_add_arg (gpg, "--with-fingerprint");
+  if (!err)
+    err = _gpgme_gpg_add_arg (gpg, "--with-fingerprint");
   if (!err)
     err = _gpgme_gpg_add_arg (gpg, secret_only ? "--list-secret-keys"
 			      : "--list-keys");
@@ -1635,7 +1712,7 @@ _gpgme_gpg_op_keylist_ext (GpgObject gpg, const char *pattern[],
     err = _gpgme_gpg_add_arg (gpg, "--");
   if (!err && pattern && *pattern)
     {
-      while (*pattern)
+      while (*pattern && **pattern)
 	err = _gpgme_gpg_add_arg (gpg, *(pattern++));
     }
 
@@ -1749,3 +1826,11 @@ _gpgme_gpg_set_io_cbs (GpgObject gpg, struct GpgmeIOCbs *io_cbs)
 {
   gpg->io_cbs = *io_cbs;
 }
+
+
+void
+_gpgme_gpg_io_event (GpgObject gpg, GpgmeEventIO type, void *type_data)
+{
+  if (gpg->io_cbs.event)
+    (*gpg->io_cbs.event) (gpg->io_cbs.event_priv, type, type_data);
+}
