@@ -1,6 +1,6 @@
 /* -*- Mode: C -*-
 
-  $Id: cryptplug.h,v 1.34 2002/06/25 17:42:21 steffen Exp $
+  $Id: cryptplug.h,v 1.41 2002/09/12 11:02:02 khz Exp $
 
   CRYPTPLUG - an independent cryptography plug-in API
 
@@ -297,6 +297,8 @@ enum {
 typedef unsigned long SigStatusFlags;
 
 
+#define CRYPTPLUG_CERT_DOES_NEVER_EXPIRE 365000
+
 
 
 
@@ -621,12 +623,18 @@ void setSignatureCertificateExpiryNearWarning( bool );
 */
 bool signatureCertificateExpiryNearWarning( void );
 
-    /*! \ingroup groupConfigSign
-      \brief Returns the number of days that are left until the
-      specified certificate expires. 
-      \param certificate the certificate to check
-    */
-    int signatureCertificateDaysLeftToExpiry( const char* certificate );
+/*! \ingroup groupConfigSign
+   \brief Returns the number of days that are left until the
+   specified certificate expires.
+   
+   Negative values show how many days ago the certificate DID expire,
+   a zero value means the certificate expires today,
+   special value CRYPTPLUG_CERT_DOES_NEVER_EXPIRE means there is
+   no expire date stored in this certificate.
+   
+   \param certificate the certificate to check
+*/
+int signatureCertificateDaysLeftToExpiry( const char* certificate );
 
 /*! \ingroup groupConfigSign
    \brief Specifies the number of days which a signature certificate must
@@ -654,12 +662,18 @@ void setCACertificateExpiryNearWarning( bool );
 */
 bool caCertificateExpiryNearWarning( void );
 
-    /*! \ingroup groupConfigSign
-      \brief Returns the number of days that are left until the
-      CA certificate of the specified certificate expires. 
-      \param certificate the certificate to check
-    */
-    int caCertificateDaysLeftToExpiry( const char* certificate );
+/*! \ingroup groupConfigSign
+  \brief Returns the number of days that are left until the
+  CA certificate of the specified certificate expires. 
+   
+   Negative values show how many days ago the certificate DID expire,
+   a zero value means the certificate expires today,
+   special value CRYPTPLUG_CERT_DOES_NEVER_EXPIRE means there is
+   no expire date stored in this certificate.
+   
+  \param certificate the certificate to check
+*/
+int caCertificateDaysLeftToExpiry( const char* certificate );
 
 /*! \ingroup groupConfigSign
    \brief Specifies the number of days which a CA certificate must
@@ -687,12 +701,18 @@ void setRootCertificateExpiryNearWarning( bool );
 */
 bool rootCertificateExpiryNearWarning( void );
 
-    /*! \ingroup groupConfigSign
-      \brief Returns the number of days that are left until the
-      root certificate of the specified certificate expires. 
-      \param certificate the certificate to check
-    */
-    int rootCertificateDaysLeftToExpiry( const char* certificate );
+/*! \ingroup groupConfigSign
+   \brief Returns the number of days that are left until the
+   root certificate of the specified certificate expires. 
+   
+   Negative values show how many days ago the certificate DID expire,
+   a zero value means the certificate expires today,
+   special value CRYPTPLUG_CERT_DOES_NEVER_EXPIRE means there is
+   no expire date stored in this certificate.
+   
+   \param certificate the certificate to check
+*/
+int rootCertificateDaysLeftToExpiry( const char* certificate );
 
 /*! \ingroup groupConfigSign
    \brief Specifies the number of days which a root certificate must
@@ -863,6 +883,11 @@ bool receiverCertificateExpiryNearWarning( void );
 /*! \ingroup groupConfigCrypt
   \brief Returns the number of days until the specified receiver
   certificate expires.
+   
+   Negative values show how many days ago the certificate DID expire,
+   a zero value means the certificate expires today,
+   special value CRYPTPLUG_CERT_DOES_NEVER_EXPIRE means there is
+   no expire date stored in this certificate.
 */
 int receiverCertificateDaysLeftToExpiry( const char* certificate );
 
@@ -911,6 +936,11 @@ int certificateInChainExpiryNearWarningInterval( void );
 /*! \ingroup groupConfigCrypt
   \brief Returns the number of days until the first certificate in
   the chain of the receiver certificate expires.
+   
+   Negative values show how many days ago the certificate DID expire,
+   a zero value means the certificate expires today,
+   special value CRYPTPLUG_CERT_DOES_NEVER_EXPIRE means there is
+   no expire date stored in this certificate.
 */
 int certificateInChainDaysLeftToExpiry( const char* certificate );
 
@@ -1518,8 +1548,9 @@ struct SignatureMetaDataExtendedInfo
     char* algo;
     char* userid;
     char* name;
-    char* email;
     char* comment;
+    char** emailList;
+    int    emailCount;
     unsigned long algo_num;
     unsigned long validity;
     unsigned long userid_num;
@@ -1575,12 +1606,16 @@ bool storeCertificatesFromMessage( const char* ciphertext );
 /*! \ingroup groupCryptAct
    \brief Find all certificate for a given addressee.
 
-   NOTE: The \c certificate parameter must point to an allready allocated
-   block of memory which is large enough to hold the complete list.
+  NOTE: The certificate parameter must point to a not-yet allocated
+        char*.  The function will allocate the memory needed and
+        return the size in newSize.
    If secretOnly is true, only secret keys are returned.
 */
-bool findCertificates( const char* addressee, char** certificates, bool secretOnly );
-
+bool findCertificates( const char* addressee,
+                       char** certificates,
+                       int* newSize,
+                       bool secretOnly );
+                       
 /*! \ingroup groupCryptAct
    \brief Encrypts an email message in
           \c cleartext according to the \c addressee and
@@ -1667,11 +1702,13 @@ bool encryptAndSignMessage( const char* cleartext,
           \c true, otherwise
           \c false.
 */
-bool decryptMessage( const char* ciphertext,
-                     bool        cipherIsBinary,
-                     int         cipherLen,
+bool decryptMessage( const char*  ciphertext,
+                     bool         cipherIsBinary,
+                     int          cipherLen,
                      const char** cleartext,
-                     const char* certificate );
+                     const char*  certificate,
+                     int* errId,
+                     char** errTxt );
 
 /*! \ingroup groupCryptAct
    \brief Combines the functionality of
@@ -1684,10 +1721,15 @@ bool decryptMessage( const char* ciphertext,
    object pointed to will contain meta information about
    the signature after the function call.
 */
-bool decryptAndCheckMessage( const char* ciphertext,
+bool decryptAndCheckMessage( const char*  ciphertext,
+                             bool         cipherIsBinary,
+                             int          cipherLen,
                              const char** cleartext,
-                             const char* certificate,
-                             struct SignatureMetaData* sigmeta );
+                             const char*  certificate,
+                             bool*        signatureFound,
+                             struct SignatureMetaData* sigmeta,
+                             int*   errId,
+                             char** errTxt );
 
 
 /*! \ingroup groupCertAct
@@ -1826,7 +1868,7 @@ struct CertificateInfo {
     dont free() it, the struct will be reused
     by the next call to nextCertificate()
   }
-  endListCertificates( it );
+  int truncated = endListCertificates( it );
 \endverbatim
 */
 struct CertIterator*
@@ -1835,17 +1877,27 @@ startListCertificates( const char* pattern, int remote );
 int
 nextCertificate( struct CertIterator*, struct CertificateInfo** result );
 
-void
+int
 endListCertificates( struct CertIterator* );
 
 /*!
   Import a certificate that was a result from a search-operation using the startListCertificates(), nextCertificate() funtions.
 
   The fingerprint must be passed to identify the key.
+
+  Additional info about the import operation is available in the additional_info parameter. The string must be free'd by the user with free().
  */
 
 int
-importCertificate( const char* fingerprint );
+importCertificateWithFPR( const char* fingerprint, char** additional_info );
+
+/*!
+  Import a certificate from memory.
+
+  Additional info about the import operation is available in the additional_info parameter. The string must be free'd by the user with free().
+*/
+int
+importCertificateFromMem( const char* data, size_t length, char** additional_info );
 
 #ifdef __cplusplus
 }
